﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using Telerik.Windows;
using System.ComponentModel;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.Scheduler;
using System.Collections.ObjectModel;

namespace HookToTimeSlotItemEvent_SL
{
    public partial class MainPage : UserControl
    {
        private IList<TimeSlotItem> timeSlotItems;
        private bool viewChanged;

        ObservableCollection<string> source = new ObservableCollection<string>{
                "item1", "item2", "item3", "item4", "item5", "item6" 
            };

        public MainPage()
        {
            this.InitializeComponent();

            this.Scheduler.Loaded += this.Scheduler_Loaded;
            this.Scheduler.ActiveViewDefinitionChanged += this.Scheduler_ActiveViewDefinitionChanged;
            this.Scheduler.LayoutUpdated += this.Scheduler_LayoutUpdated;
            this.Scheduler.View.PropertyChanged += this.OnSchedulerViewPropertyChanged;

            RadDragAndDropManager.AddDragQueryHandler(this, OnDragQuery);
            RadDragAndDropManager.AddDropQueryHandler(this, OnDropQuery);
            RadDragAndDropManager.AddDropInfoHandler(this, OnDropInfo);

            this.listBox.ItemsSource = this.source;
        }

        protected virtual void OnDragQuery(object sender, DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
        }

        void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
        }

        private void OnDropInfo(object sender, DragDropEventArgs e)
        {
            var sourceListBoxItem = e.Options.Source as System.Windows.Controls.ListBoxItem;
            var timeSlotItem = e.Options.Destination as TimeSlotItem;
            if (e.Options.Status == DragStatus.DropComplete && timeSlotItem != null)
            {
                this.Scheduler.Appointments.Add(
                    new Appointment()
                    {
                        Start = timeSlotItem.TimeSlot.Start,
                        End = timeSlotItem.TimeSlot.End,
                        Subject = sourceListBoxItem.FindChildByType<TextBlock>().Text
                    }
                );
                this.source.Remove(sourceListBoxItem.Content.ToString());
            }
        }

        private void InitializeTimeSlotItems()
        {
            this.timeSlotItems = this.Scheduler.ChildrenOfType<TimeSlotItem>();
            foreach (TimeSlotItem item in this.timeSlotItems)
            {
                item.SetValue(RadDragAndDropManager.AllowDropProperty, true);
            }
        }

        private void OnSchedulerViewPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "VisibleRangeStart" || e.PropertyName == "VisibleRangeEnd")
            {
                this.viewChanged = true;
            }
        }

        private void Scheduler_ActiveViewDefinitionChanged(object sender, EventArgs e)
        {
            this.viewChanged = true;
        }

        private void Scheduler_LayoutUpdated(object sender, EventArgs e)
        {
            if (this.viewChanged)
            {
                this.viewChanged = false;
                this.InitializeTimeSlotItems();
            }
        }

        private void Scheduler_Loaded(object sender, RoutedEventArgs e)
        {
            this.InitializeTimeSlotItems();
            this.viewChanged = true;
        }
    }
}
